use drm::control::Device;

mod card;

fn main() {
  let gpu = match card::Card::open("/dev/dri/card1") {
    Ok(gpu) => gpu,
    Err(err) => {
      panic!("Impossible to open DRM file : {:?}", err);
    }
  };

  println!("{:?}", gpu);
  print_connectors(&gpu);
}

fn print_connectors(gpu: &card::Card) {
  let resource_handles = gpu.resource_handles().unwrap();

  println!("Connectors:");
  for handle in resource_handles.connectors() {
    print_connector(&gpu, handle);
  }
}

fn print_connector(gpu: &card::Card, connector_handle: &drm::control::connector::Handle) {
  match gpu.get_connector(*connector_handle, false) {
    Ok(connector) => {
      println!("\t{} ({:?})", connector.interface().as_str(), connector.state());

      let encoder_info = format_encoder(&gpu, connector.current_encoder());

      println!("\t\tEncoder: {}", encoder_info);
      println!(""); // Add blank line
    },
    Err(_) => {}
  };
}

fn format_encoder(gpu: &card::Card, encoder_handle: Option<drm::control::encoder::Handle>) -> String {
  match encoder_handle.and_then(|handle| gpu.get_encoder(handle).ok()) {
    Some(encoder) => {
      let crtc_info = format_crtc(&gpu, encoder.crtc());

      format!("{:?}, {}", encoder.kind(), crtc_info)
    }
    None => format!("No encoder found")
  }
}

fn format_crtc(gpu: &card::Card, crtc_handle: Option<drm::control::crtc::Handle>) -> String {
  match crtc_handle.and_then(|handle| Some(gpu.get_crtc(handle))) {
    Some(crtc) => format!("{:?}", crtc),
    None => format!("No CRTC data")
  }
}
